////////////////////////////////////////////////////////////////////////////////
//  Project: Pong Game Clone
//  Author: Vladeta Stojanovic (vladeta_stojanovic@yahoo.co.uk)
//  File: main.cpp
//  Description: This file contains all of the main game code and the implementation of
//              gameplay realted functions.
//  Version: v0.1
//
//  Date (last revision): March 16, 2009
//  Date (creation): March 30, 2009
//
//
//  Revision history notes:
//  ---Date---|---Revision---|---Revisor---|
//
//  Credits:  Portions of this code were based on the examples form the book
//            "Programming The Nintendo Game Boy Advance: The Unofficial Guide"
//            by Jonathan S. Harbour (http://www.jharbour.com).
//            Paddle collision detection code based on the Pong game code
//            by Aaron Cox (http://www.aaroncox.net).
//            Additional code and examples are derived from the code presented
//            in TONC v1.4 by J Vijn (http://www.coranac.com/projects/#tonc).
//
//  License: You are allowed to use the code in this file for presonal,
//           scholarship and/or non-commercial purpuses. This program code
//           comes with NO WARRANTY.
//
///////////////////////////////////////////////////////////////////////////////

#include "defs.hpp"
#include "core.hpp"

//include the sprite data
//include all the sprite data
#include "ball.h"
#include "paddle.h"

//background picture includes
#include "bitmap.raw.c"
#include "bitmap.pal.c"

//game functions
//int update_score();
void new_game();
void move_ball();
void move_paddle();
bool paddle_ball_collision(int sprite_num);
void handle_ball();
void handle_paddle_ai();

int main(void)
{
    int i, j;
    
    //set video mode to mode 3, background 2, with sprite support
    SetMode(4 | OBJ_ENABLE | OBJ_MAP_1D | BG2_ENABLE);
    
    //set up the palette colors
    for (i = 0; i < 256; ++i)
    {
        BGPaletteMem[i] =  bitmap_Palette[i];
    }

    //set the ball sprite palette
    for(i = 0; i < 256; ++i)
    {
        SpritePal[i] = ballPalette[i];
    }

    //load in the sprite and background data
    memcpy(video_buffer, bitmap_Bitmap, 38400);
    memcpy(SpriteData3, ballData, 1024);
    memcpy(&(SpriteData3[1024]), paddleData, 2048);

    //hide all the loaded sprites
    hide_sprites();

    //intialize the ball sprite
    for(i = 0; i < NUM_BALLS; ++i)
    {
        init_sprites(i, false, rand() % 230, rand() % 150, 32, COLOR_256, 512);

        while(game_sprites[i].dir_x == 0)
        {
            game_sprites[i].dir_x = rand() % 6 - 3;
        }
        while(game_sprites[i].dir_y == 0)
        {
            game_sprites[i].dir_y = rand() % 6 - 3;
        }
    }

    //initialize the paddle sprites
    for(j = 0; j < NUM_PADDLES; ++j)
    {
        init_sprites(j+NUM_BALLS, true, 15 + (j * 180), 30, 64, COLOR_256, 512 + 64);
    }
    
    while(1)
    {
        //wait for vertical retrace
        vsync();
        
        //check for button presses
        check_buttons();

        // if the game is not paused
        if (!button_pressed(BUTTON_A)) //TODO: Make it so that the button pressed needs to pressed only once
        {
            
            //check that no one has scored yet
            new_game();
            
            //move the ball
            move_ball();
            
            //update the ai
            handle_paddle_ai();
            
            //move the paddle
            move_paddle();
            
            //display the sprite
            update_oam();

            //call game timer
            timer (30000);
        }
    }
}

void new_game()
{
    if ((game_sprites[0].x >= 208) || (game_sprites[0].x <= 32))
    {
        //if the ball tocuhes the wall behind either of the paddles
        //reset player paddle, ai paddle and coords
        game_sprites[0].x = 240 / 2;
        game_sprites[0].y = 160 /2;
        game_sprites[1].x = 15;
        game_sprites[1].y = 30;
        game_sprites[2].x = 195;
        game_sprites[2].y = 30;
    }
}

void move_ball()
{
    int i;
    
    //ball move
    for (i = 0; i < NUM_BALLS; ++i)
    {
        handle_ball();

        //update sprite x position
        game_sprites[i].x += game_sprites[i].dir_x;
        if (game_sprites[i].x > 239 - game_sprites[i].size)
        {
            game_sprites[i].x = 239 - game_sprites[i].size;
            game_sprites[i].dir_x *= -1;
        }
        if (game_sprites[i].x < 1)
        {
            game_sprites[i].x = 1;
            game_sprites[i].dir_x *= -1;
        }

        //update sprite y position
        game_sprites[i].y += game_sprites[i].dir_y;
        if (game_sprites[i].y + game_sprites[i].size > 159)
        {
            game_sprites[i].y = 159 - game_sprites[i].size;
            game_sprites[i].dir_y *= -1;
        }
        if (game_sprites[i].y < 1)
        {
            game_sprites[i].y = 1;
            game_sprites[i].dir_y *= -1;
        }

        //update the sprite properties
        move_sprite(i);

    }
}

void move_paddle()
{
    int j = NUM_BALLS;
    
    if(button_pressed(BUTTON_DOWN))
    {
        game_sprites[j].y += game_sprites[j].dir_y;

        //check collision at the bottom of the screen
            if (game_sprites[j].y > (159 - 57))
            {
                game_sprites[j].y = 159 - 57;
            }
        }

        if(button_pressed(BUTTON_UP))
        {
            game_sprites[j].y -= game_sprites[j].dir_y;

            //check collision at the top of the screen
            if (game_sprites[j].y < 0)
            {
                game_sprites[j].y = 0;
            }
        }

        //update the sprite properties
        move_sprite(j);
}


bool paddle_ball_collision(int sprite_num)
{
    //temp collision varaibles
    int ball_x = game_sprites[0].x;
    int ball_y = game_sprites[0].y;
    int ball_width = game_sprites[0].size;
    int ball_height = game_sprites[0].size;
    int ball_speed = BALL_X_Y_SPEED;

    int paddle_x = game_sprites[sprite_num].x;
    int paddle_y = game_sprites[sprite_num].y;
    int paddle_width = 32;
    int paddle_height = 64;
    
    if(sprite_num == 1)
    {
        //collision for player paddle
        if ((ball_speed > 0) && (ball_y + ball_height >= paddle_y) && (ball_y <= paddle_y + paddle_height))
        {
            if ((ball_x <= paddle_x + paddle_width) && (ball_x + ball_width >= paddle_x))
            {
                return true;
            }
        }
    }
    else
    {
        //collision for ai paddle
        if ((ball_speed > 0) && (ball_y + ball_height >= paddle_y) && (ball_y <= paddle_y + paddle_height))
        {
            if ((ball_x <= paddle_x + paddle_width) && (ball_x + ball_width >= paddle_x))
            {
                return true;
            }
        }
    }
    
    return false;
}

void handle_ball()
{
    for (int i = NUM_BALLS;i < NUM_BALLS + 2; ++i)
    {
        if (paddle_ball_collision(i) == true)
        {
            if(game_sprites[0].x > 120)
            {
                game_sprites[0].dir_x = -BALL_X_Y_SPEED;
            }
            else
            {
                 game_sprites[0].dir_x = BALL_X_Y_SPEED;
            }
        }
    }
}

void handle_paddle_ai()
{
    //TODO: Improve AI
    int ai_speed = PADDLE_Y_SPEED; //add more randomness
    //if(game_sprites[0].x > (88 + rand() % 8))

    //if ball is above the paddle, move the paddle up
    if(game_sprites[0].y + game_sprites[0].size < game_sprites[2].y)
    {
         game_sprites[2].y -= ai_speed;
    }
    
    //if ball is below the paddle, move the paddle down
    if(game_sprites[0].y > game_sprites[2].y + game_sprites[2].size)
    {
         game_sprites[2].y += ai_speed;
    }
    
    //wall collision
    //check collision at the bottom of the screen
    if (game_sprites[2].y > (159 - 57))
    {
        game_sprites[2].y = 159 - 57;
    }
    
     //check collision at the top of the screen
     if (game_sprites[2].y < 0)
     {
        game_sprites[2].y = 0;
     }
    
    //update the sprite properties
    move_sprite(2);
    
}

